'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var image = {
  name: 'vue-coe-image',
  props: {
    loaderImage: {
      type: String,
      default: 'https://cdn-images-1.medium.com/max/1600/1*9EBHIOzhE1XfMYoKz1JcsQ.gif'
    },
    src: {
      type: [String, File],
      required: true
    },
    srcset: String,
    fallback: {
      type: String
    },
    animation: {
      type: Boolean,
      default: true
    },
    delay: {
      type: [String, Number],
      default: 0
    },
    blurLevel: {
      type: Number,
      default: 30
    },
    duration: {
      type: Number,
      default: 500
    },
    intersectionOptions: {
      type: Object,
      default: function _default() {
        return {};
      }
    }
  },
  data: function data() {
    return {
      image: null,
      imageHandler: null,
      rate: 1,
      observer: null,
      intersected: false,
      hasError: false
    };
  },
  watch: {
    src: 'setSource',
    hasError: 'setFallback'
  },
  created: function created() {
    var _this = this;

    this.image = this.loaderImage;
    setTimeout(function () {
      _this.observer = new IntersectionObserver(function (targets) {
        var image = targets[0];

        if (image.isIntersecting) {
          _this.intersected = true;

          _this.setImage();

          _this.observer.disconnect();

          _this.$emit('intersect');
        }
      }, _this.intersectionOptions);

      _this.observer.observe(_this.$el);
    }, this.delay);
  },
  computed: {
    deviation: function deviation() {
      return this.blurLevel * this.rate;
    }
  },
  methods: {
    setSource: function setSource(x, y) {
      if (x !== y) {
        this.hasError = false;
        this.setImage();
      }
    },
    setFallback: function setFallback(status) {
      if (status) this.image = this.fallback;
    },
    handleLoad: function handleLoad() {
      this.image = this.imageHandler.src;
      this.animation && this.animate();
    },
    handleError: function handleError() {
      if (process.env.NODE_ENV !== 'production' && !this.fallback) {
        console.warn('an error occured during the image loading');
      }

      this.hasError = true;
      this.$emit('error');
    },
    setImage: function setImage() {
      this.imageHandler = new Image();
      this.imageHandler.src = this.src || this.placeholder;
      this.imageHandler.onload = this.handleLoad;
      this.imageHandler.onerror = this.handleError;
    },
    animate: function animate() {
      var _this2 = this;

      var start = Date.now() + this.duration;

      var step = function step() {
        var remaining = start - Date.now();

        if (remaining < 0) {
          _this2.rate = 0;
        } else {
          _this2.rate = remaining / _this2.duration;
          requestAnimationFrame(step);
        }
      };

      requestAnimationFrame(step);
    }
  },
  render: function render(h) {
    var feGaussianBlur = h('feGaussianBlur', {
      attrs: {
        in: 'SourceGraphic',
        stdDeviation: this.deviation
      }
    });
    var filter = h('filter', {
      attrs: {
        id: 'blur'
      }
    }, [feGaussianBlur]);
    var defs = h('defs', null, [filter]);
    var svg = h('svg', {
      class: ['filter-defs-svg hidden'],
      attrs: {
        xmlns: 'http://www.w3.org/2000/svg',
        version: '1.1'
      }
    }, [defs]);
    var image = h('img', {
      style: "url(".concat(this.image, ")"),
      attrs: {
        src: this.image
      },
      class: 'lazy-load-image',
      ref: 'img'
    });
    return h('div', {
      class: 'vue-coe-image'
    }, [!this.hasError && this.intersected && svg, image]);
  },
  beforeDestroy: function beforeDestroy() {
    this.observer.unobserve(this.$refs.img);
  }
};
var VueCoeImage = {
  install: function install(Vue) {
    Vue.component('VueCoeImage', image);
  }
};

exports.default = image;
exports.VueCoeImage = VueCoeImage;
